/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// sbar.c -- status bar code

#include "quakedef.h"


#define STAT_MINUS		10	// num frame for '-' stats digit
qpic_t		*sb_nums[2][11];
qpic_t		*sb_colon, *sb_slash;
qpic_t		*sb_ibar;
qpic_t		*sb_sbar;
qpic_t		*sb_scorebar;

qpic_t      *sb_weapons[7][8];   // 0 is active, 1 is owned, 2-5 are flashes
qpic_t      *sb_ammo[4];
qpic_t		*sb_sigil[4];
qpic_t		*sb_armor[3];
qpic_t		*sb_items[32];

qpic_t	*sb_faces[7][2];		// 0 is gibbed, 1 is dead, 2-6 are alive
							// 0 is static, 1 is temporary animation
qpic_t	*sb_face_invis;
qpic_t	*sb_face_quad;
qpic_t	*sb_face_invuln;
qpic_t	*sb_face_invis_invuln;

qboolean	sb_showscores;

qpic_t      *rsb_invbar[2];
qpic_t      *rsb_weapons[5];
qpic_t      *rsb_items[2];
qpic_t      *rsb_ammo[3];
qpic_t      *rsb_teambord;		// PGM 01/19/97 - team color border

//MED 01/04/97 added two more weapons + 3 alternates for grenade launcher
qpic_t      *hsb_weapons[7][5];   // 0 is active, 1 is owned, 2-5 are flashes
//MED 01/04/97 added array to simplify weapon parsing
int         hipweapons[4] = {HIT_LASER_CANNON_BIT,HIT_MJOLNIR_BIT,4,HIT_PROXIMITY_GUN_BIT};
//MED 01/04/97 added hipnotic items array
qpic_t      *hsb_items[2];

void Sbar_MiniDeathmatchOverlay (void);
void Sbar_DeathmatchOverlay (void);
void M_DrawPic (int x, int y, qpic_t *pic);
void Sbar_Load3DWidgets (void);

int SbarOffset;


/*
===============
Sbar_ShowScores

Tab key down
===============
*/
void Sbar_ShowScores (void)
{
	sb_showscores = !sb_showscores;
}

/*
===============
Sbar_DontShowScores

Tab key up
===============
*/
void Sbar_DontShowScores (void)
{
}


/*
===============
Sbar_Init
===============
*/
void Sbar_Init (void)
{
	int		i;

	for (i=0 ; i<10 ; i++)
	{
		sb_nums[0][i] = Draw_PicFromWad (va("num_%i",i));
		sb_nums[1][i] = Draw_PicFromWad (va("anum_%i",i));
	}

	sb_nums[0][10] = Draw_PicFromWad ("num_minus");
	sb_nums[1][10] = Draw_PicFromWad ("anum_minus");

	sb_colon = Draw_PicFromWad ("num_colon");
	sb_slash = Draw_PicFromWad ("num_slash");

	sb_weapons[0][0] = Draw_PicFromWad ("inv_shotgun");
	sb_weapons[0][1] = Draw_PicFromWad ("inv_sshotgun");
	sb_weapons[0][2] = Draw_PicFromWad ("inv_nailgun");
	sb_weapons[0][3] = Draw_PicFromWad ("inv_snailgun");
	sb_weapons[0][4] = Draw_PicFromWad ("inv_rlaunch");
	sb_weapons[0][5] = Draw_PicFromWad ("inv_srlaunch");
	sb_weapons[0][6] = Draw_PicFromWad ("inv_lightng");

	sb_weapons[1][0] = Draw_PicFromWad ("inv2_shotgun");
	sb_weapons[1][1] = Draw_PicFromWad ("inv2_sshotgun");
	sb_weapons[1][2] = Draw_PicFromWad ("inv2_nailgun");
	sb_weapons[1][3] = Draw_PicFromWad ("inv2_snailgun");
	sb_weapons[1][4] = Draw_PicFromWad ("inv2_rlaunch");
	sb_weapons[1][5] = Draw_PicFromWad ("inv2_srlaunch");
	sb_weapons[1][6] = Draw_PicFromWad ("inv2_lightng");

	for (i=0 ; i<5 ; i++)
	{
		sb_weapons[2+i][0] = Draw_PicFromWad (va("inva%i_shotgun",i+1));
		sb_weapons[2+i][1] = Draw_PicFromWad (va("inva%i_sshotgun",i+1));
		sb_weapons[2+i][2] = Draw_PicFromWad (va("inva%i_nailgun",i+1));
		sb_weapons[2+i][3] = Draw_PicFromWad (va("inva%i_snailgun",i+1));
		sb_weapons[2+i][4] = Draw_PicFromWad (va("inva%i_rlaunch",i+1));
		sb_weapons[2+i][5] = Draw_PicFromWad (va("inva%i_srlaunch",i+1));
		sb_weapons[2+i][6] = Draw_PicFromWad (va("inva%i_lightng",i+1));
	}

	sb_ammo[0] = Draw_PicFromWad ("sb_shells");
	sb_ammo[1] = Draw_PicFromWad ("sb_nails");
	sb_ammo[2] = Draw_PicFromWad ("sb_rocket");
	sb_ammo[3] = Draw_PicFromWad ("sb_cells");

	sb_armor[0] = Draw_PicFromWad ("sb_armor1");
	sb_armor[1] = Draw_PicFromWad ("sb_armor2");
	sb_armor[2] = Draw_PicFromWad ("sb_armor3");

	sb_items[0] = Draw_PicFromWad ("sb_key1");
	sb_items[1] = Draw_PicFromWad ("sb_key2");
	sb_items[2] = Draw_PicFromWad ("sb_invis");
	sb_items[3] = Draw_PicFromWad ("sb_invuln");
	sb_items[4] = Draw_PicFromWad ("sb_suit");
	sb_items[5] = Draw_PicFromWad ("sb_quad");

	sb_sigil[0] = Draw_PicFromWad ("sb_sigil1");
	sb_sigil[1] = Draw_PicFromWad ("sb_sigil2");
	sb_sigil[2] = Draw_PicFromWad ("sb_sigil3");
	sb_sigil[3] = Draw_PicFromWad ("sb_sigil4");

	sb_faces[4][0] = Draw_PicFromWad ("face1");
	sb_faces[4][1] = Draw_PicFromWad ("face_p1");
	sb_faces[3][0] = Draw_PicFromWad ("face2");
	sb_faces[3][1] = Draw_PicFromWad ("face_p2");
	sb_faces[2][0] = Draw_PicFromWad ("face3");
	sb_faces[2][1] = Draw_PicFromWad ("face_p3");
	sb_faces[1][0] = Draw_PicFromWad ("face4");
	sb_faces[1][1] = Draw_PicFromWad ("face_p4");
	sb_faces[0][0] = Draw_PicFromWad ("face5");
	sb_faces[0][1] = Draw_PicFromWad ("face_p5");

	sb_face_invis = Draw_PicFromWad ("face_invis");
	sb_face_invuln = Draw_PicFromWad ("face_invul2");
	sb_face_invis_invuln = Draw_PicFromWad ("face_inv2");
	sb_face_quad = Draw_PicFromWad ("face_quad");

	Cmd_AddCommand ("+showscores", Sbar_ShowScores);
	Cmd_AddCommand ("-showscores", Sbar_DontShowScores);

	sb_sbar = Draw_PicFromWad ("sbar");
	sb_ibar = Draw_PicFromWad ("ibar");
	sb_scorebar = Draw_PicFromWad ("scorebar");

//MED 01/04/97 added new hipnotic weapons
	if (hipnotic)
	{
	  hsb_weapons[0][0] = Draw_PicFromWad ("inv_laser");
	  hsb_weapons[0][1] = Draw_PicFromWad ("inv_mjolnir");
	  hsb_weapons[0][2] = Draw_PicFromWad ("inv_gren_prox");
	  hsb_weapons[0][3] = Draw_PicFromWad ("inv_prox_gren");
	  hsb_weapons[0][4] = Draw_PicFromWad ("inv_prox");

	  hsb_weapons[1][0] = Draw_PicFromWad ("inv2_laser");
	  hsb_weapons[1][1] = Draw_PicFromWad ("inv2_mjolnir");
	  hsb_weapons[1][2] = Draw_PicFromWad ("inv2_gren_prox");
	  hsb_weapons[1][3] = Draw_PicFromWad ("inv2_prox_gren");
	  hsb_weapons[1][4] = Draw_PicFromWad ("inv2_prox");

	  for (i=0 ; i<5 ; i++)
	  {
		 hsb_weapons[2+i][0] = Draw_PicFromWad (va("inva%i_laser",i+1));
		 hsb_weapons[2+i][1] = Draw_PicFromWad (va("inva%i_mjolnir",i+1));
		 hsb_weapons[2+i][2] = Draw_PicFromWad (va("inva%i_gren_prox",i+1));
		 hsb_weapons[2+i][3] = Draw_PicFromWad (va("inva%i_prox_gren",i+1));
		 hsb_weapons[2+i][4] = Draw_PicFromWad (va("inva%i_prox",i+1));
	  }

	  hsb_items[0] = Draw_PicFromWad ("sb_wsuit");
	  hsb_items[1] = Draw_PicFromWad ("sb_eshld");
	}

	if (rogue)
	{
		rsb_invbar[0] = Draw_PicFromWad ("r_invbar1");
		rsb_invbar[1] = Draw_PicFromWad ("r_invbar2");

		rsb_weapons[0] = Draw_PicFromWad ("r_lava");
		rsb_weapons[1] = Draw_PicFromWad ("r_superlava");
		rsb_weapons[2] = Draw_PicFromWad ("r_gren");
		rsb_weapons[3] = Draw_PicFromWad ("r_multirock");
		rsb_weapons[4] = Draw_PicFromWad ("r_plasma");

		rsb_items[0] = Draw_PicFromWad ("r_shield1");
        rsb_items[1] = Draw_PicFromWad ("r_agrav1");

// PGM 01/19/97 - team color border
        rsb_teambord = Draw_PicFromWad ("r_teambord");
// PGM 01/19/97 - team color border

		rsb_ammo[0] = Draw_PicFromWad ("r_ammolava");
		rsb_ammo[1] = Draw_PicFromWad ("r_ammomulti");
		rsb_ammo[2] = Draw_PicFromWad ("r_ammoplasma");
	}

	// now load the 3D widgets
	Sbar_Load3DWidgets ();
}


/*
==============================================================================================================================

		3D WIDGETS

==============================================================================================================================
*/

// widget display list numbers
int HUD_Weaps[10];
int HUD_Ammo[7];
int HUD_Face;
int HUD_Arm[3];
int HUD_Quad;
int HUD_Invis;
int HUD_Invul;
int HUD_Suit;
int HUD_BitModels[10];

float bobjrotate;

void R_DrawSbarWidgetModel (model_t *wm, int skinnum);
void R_DrawSbarBrushModel (model_t *mod);

/*
================
Sbar_LoadWidgetModel

Loads a widget model and caches in a display list.
Funky stuff (tm) is done to the model to squash it's verts into the desired range for rendering.
================
*/
int Sbar_LoadWidgetModel (char *name, int skinnum)
{
	model_t *wm;
	int TheDL = 0;

	// these are all standard quake models
	// if it loads, well and good.  if it don't, replace it with the standard icon
	wm = Mod_ForName (name, false);

	if (!wm)
	{
		Con_DPrintf ("Failed to load %s\n", name);
		return 0;
	}

	// now cache it in a display list
	TheDL = glGenLists (1);

	glNewList (TheDL, GL_COMPILE);

	// call the appropriate drawing routine
	// we don't expect sprites to ever come in here
	if (wm->type == mod_alias)
		R_DrawSbarWidgetModel (wm, skinnum);
	else if (wm->type == mod_brush)
		R_DrawSbarBrushModel (wm);

	glEndList ();

	// return the display list number
	return TheDL;
}


void Sbar_Load3DWidgets (void)
{
	// player "face"
	HUD_Face = Sbar_LoadWidgetModel ("maps/b_bh100.bsp", 0);

	// armour
	// rogue has it's own armour mdls but - hey, guess what!!! - they're actually the same...
	HUD_Arm[0] = Sbar_LoadWidgetModel ("progs/armor.mdl", 0);
	HUD_Arm[1] = Sbar_LoadWidgetModel ("progs/armor.mdl", 1);
	HUD_Arm[2] = Sbar_LoadWidgetModel ("progs/armor.mdl", 2);

	// powerups
	HUD_Quad = Sbar_LoadWidgetModel ("progs/quaddama.mdl", 0);
	HUD_Invis = Sbar_LoadWidgetModel ("progs/invisibl.mdl", 0);
	HUD_Invul = Sbar_LoadWidgetModel ("progs/invulner.mdl", 0);
	HUD_Suit = Sbar_LoadWidgetModel ("progs/suit.mdl", 0);

	// bits of stuff
	HUD_BitModels[0] = Sbar_LoadWidgetModel ("progs/end1.mdl", 0);
	HUD_BitModels[1] = Sbar_LoadWidgetModel ("progs/end2.mdl", 0);
	HUD_BitModels[2] = Sbar_LoadWidgetModel ("progs/end3.mdl", 0);
	HUD_BitModels[3] = Sbar_LoadWidgetModel ("progs/end4.mdl", 0);
	HUD_BitModels[4] = Sbar_LoadWidgetModel ("progs/b_s_key.mdl", 0);
	HUD_BitModels[5] = Sbar_LoadWidgetModel ("progs/b_g_key.mdl", 0);
	HUD_BitModels[6] = Sbar_LoadWidgetModel ("progs/m_s_key.mdl", 0);
	HUD_BitModels[7] = Sbar_LoadWidgetModel ("progs/m_g_key.mdl", 0);
	HUD_BitModels[8] = Sbar_LoadWidgetModel ("progs/w_s_key.mdl", 0);
	HUD_BitModels[9] = Sbar_LoadWidgetModel ("progs/w_g_key.mdl", 0);

	// ammo boxes
	// normal quake
	HUD_Ammo[0] = Sbar_LoadWidgetModel ("maps/b_shell1.bsp", 0);
	HUD_Ammo[1] = Sbar_LoadWidgetModel ("maps/b_nail1.bsp", 0);
	HUD_Ammo[2] = Sbar_LoadWidgetModel ("maps/b_rock0.bsp", 0);
	HUD_Ammo[3] = Sbar_LoadWidgetModel ("maps/b_batt1.bsp", 0);
	// rogue
	HUD_Ammo[4] = Sbar_LoadWidgetModel ("maps/b_lnail1.bsp", 0);
	HUD_Ammo[5] = Sbar_LoadWidgetModel ("maps/b_plas0.bsp", 0);	// this looks nice - just to show that i can do non-cubes
	HUD_Ammo[6] = Sbar_LoadWidgetModel ("maps/b_mrock0.bsp", 0);
	// hipnotic has no ammo box models
}


//=============================================================================

// drawing routines are relative to the status bar location

/*
=============
Sbar_DrawPic
=============
*/
void Sbar_DrawPic (int x, int y, qpic_t *pic)
{
	if (cl.gametype == GAME_DEATHMATCH)
		Draw_Pic (x /* + ((vid.width2D - 320)>>1)*/, y + (vid.height2D-SBAR_HEIGHT), pic);
	else
		Draw_Pic (x + ((vid.width2D - 320)>>1), y + (vid.height2D-SBAR_HEIGHT), pic);
}

/*
=============
Sbar_DrawTransPic
=============
*/
void Sbar_DrawTransPic (int x, int y, qpic_t *pic)
{
	if (cl.gametype == GAME_DEATHMATCH)
		Draw_TransPic (x /*+ ((vid.width2D - 320)>>1)*/, y + (vid.height2D-SBAR_HEIGHT), pic);
	else
		Draw_TransPic (x + ((vid.width2D - 320)>>1), y + (vid.height2D-SBAR_HEIGHT), pic);
}

/*
================
Sbar_DrawCharacter

Draws one solid graphics character
================
*/
void Sbar_DrawCharacter (int x, int y, int num)
{
	if (cl.gametype == GAME_DEATHMATCH)
		Draw_Character ( x /*+ ((vid.width2D - 320)>>1) */ + 4 , y + vid.height2D-SBAR_HEIGHT, num);
	else
		Draw_Character ( x + ((vid.width2D - 320)>>1) + 4 , y + vid.height2D-SBAR_HEIGHT, num);
}

/*
================
Sbar_DrawString
================
*/
void Sbar_DrawString (int x, int y, char *str)
{
	if (cl.gametype == GAME_DEATHMATCH)
		Draw_String (x /*+ ((vid.width2D - 320)>>1)*/, y+ vid.height2D-SBAR_HEIGHT, str);
	else
		Draw_String (x + ((vid.width2D - 320)>>1), y+ vid.height2D-SBAR_HEIGHT, str);
}

/*
=============
Sbar_itoa
=============
*/
int Sbar_itoa (int num, char *buf)
{
	char	*str;
	int		pow10;
	int		dig;

	str = buf;

	if (num < 0)
	{
		*str++ = '-';
		num = -num;
	}

	for (pow10 = 10 ; num >= pow10 ; pow10 *= 10)
	;

	do
	{
		pow10 /= 10;
		dig = num/pow10;
		*str++ = '0'+dig;
		num -= dig*pow10;
	} while (pow10 != 1);

	*str = 0;

	return str-buf;
}


/*
=============
Sbar_DrawNum
=============
*/
void Sbar_DrawNum (int x, int y, int num, int digits, int color)
{
	char			str[12];
	char			*ptr;
	int				l, frame;

	// bound number so it doesn't overflow
	if (num > 666) num = 666;
	if (num < -99) num = -99;

	sprintf (str, "%i", num);
	ptr = str;

	if (num > 99)
		;	// 3 digits
	else if (num < -9)
		;	// 3 digits
	else if (num > 9)
		x += 24;	// 2 digits
	else if (num >= 0)
		x += 48;	// 1 digit
	else if (num < 0)
		x += 24;	// 1 digit

	while (*ptr)
	{
		if (*ptr == '-')
			frame = STAT_MINUS;
		else
			frame = *ptr -'0';

		Draw_TransPic (x, y, sb_nums[color][frame]);
		x += 24;
		ptr++;
	}
}


//=============================================================================

int		fragsort[MAX_SCOREBOARD];

char	scoreboardtext[MAX_SCOREBOARD][20];
int		scoreboardtop[MAX_SCOREBOARD];
int		scoreboardbottom[MAX_SCOREBOARD];
int		scoreboardcount[MAX_SCOREBOARD];
int		scoreboardlines;

/*
===============
Sbar_SortFrags
===============
*/
void Sbar_SortFrags (void)
{
	int		i, j, k;

	// sort by frags
	scoreboardlines = 0;

	for (i = 0; i < cl.maxclients; i++)
	{
		if (cl.scores[i].name[0])
		{
			fragsort[scoreboardlines] = i;
			scoreboardlines++;
		}
	}

	for (i = 0; i < scoreboardlines; i++)
	{
		for (j = 0; j < scoreboardlines - 1 - i; j++)
		{
			// sort descending cos we're going to draw from the bottom up
			if (cl.scores[fragsort[j]].frags > cl.scores[fragsort[j + 1]].frags)
			{
				k = fragsort[j];
				fragsort[j] = fragsort[j + 1];
				fragsort[j + 1] = k;
			}
		}
	}
}

int	Sbar_ColorForMap (int m)
{
	return m < 128 ? m + 8 : m + 8;
}

/*
===============
Sbar_UpdateScoreboard
===============
*/
void Sbar_UpdateScoreboard (void)
{
	int		i, k;
	int		top, bottom;
	scoreboard_t	*s;

	Sbar_SortFrags ();

	// draw the text
	memset (scoreboardtext, 0, sizeof(scoreboardtext));

	for (i = 0; i < scoreboardlines; i++)
	{
		k = fragsort[i];
		s = &cl.scores[k];

		sprintf (&scoreboardtext[i][1], "%3i %s", s->frags, s->name);

		top = s->colors & 0xf0;
		bottom = (s->colors & 15) <<4;
		scoreboardtop[i] = Sbar_ColorForMap (top);
		scoreboardbottom[i] = Sbar_ColorForMap (bottom);
	}
}



/*
===============
Sbar_SoloScoreboard
===============
*/
void Sbar_SoloScoreboard (void)
{
	char str[80];
	int minutes, seconds, tens, units;
	int l;
	int i;
	int SBX;
	int SBY;

	// calculate time
	minutes = cl.time / 60;
	seconds = cl.time - 60 * minutes;
	tens = seconds / 10;
	units = seconds - 10 * tens;

	// base X
	SBX = vid.width2D / 2;

	// starting Y
	SBY = vid.height2D * 0.35;

	// level name
	l = strlen (cl.levelname);
	Draw_String (SBX - l * 4, SBY, cl.levelname);

	for (i = 0; i < l; i++)
	{
		if (!i)
			str[i] = 32;
		else if (i == 1)
			str[i] = 29;
		else if (i == l - 2)
			str[i] = 31;
		else if (i == l - 1)
			str[i] = 32;
		else str[i] = 30;
	}

	str[l] = 0;
	SBY += 12;
	Draw_String (SBX - l * 4, SBY, str);

	// kill count
	sprintf (str, "Monsters: %i/%i", cl.stats[STAT_MONSTERS], cl.stats[STAT_TOTALMONSTERS]);
	l = strlen (str);
	SBY += 12;
	Draw_String (SBX - l * 4, SBY, str);

	// secrets count
	sprintf (str, "Secrets: %i/%i", cl.stats[STAT_SECRETS], cl.stats[STAT_TOTALSECRETS]);
	l = strlen (str);
	SBY += 12;
	Draw_String (SBX - l * 4, SBY, str);

	// time elapsed
	sprintf (str, "Time: %i:%i%i", minutes, tens, units);
	l = strlen (str);
	SBY += 12;
	Draw_String (SBX - l * 4, SBY, str);

	// skill
	switch ((int) skill.value)
	{
	case 0:
		strcpy (str, "Skill: Easy");
		break;

	case 1:
		strcpy (str, "Skill: Normal");
		break;

	case 2:
		strcpy (str, "Skill: Hard");
		break;

	case 3:
		strcpy (str, "Skill: NIGHTMARE");
		break;
	}

	l = strlen (str);
	SBY += 12;
	Draw_String (SBX - l * 4, SBY, str);
}


/*
===============
Sbar_DrawScoreboard
===============
*/
void Sbar_DrawScoreboard (void)
{
	if (cl.gametype == GAME_DEATHMATCH)
		Sbar_DeathmatchOverlay ();
	else Sbar_SoloScoreboard ();
}

//=============================================================================

/*
===============
HUD_DrawWeaponses

I can't do my 3D HUD with these cos of the need to properly support Hipnotic's STUPID FUCKING HAMMER,
so let's do something that's at least halfway decent looking with the icons.
===============
*/
void HUD_DrawWeaponses (void)
{
	int		i;
	char	num[6];
	float	time;
	int		flashon;
	int		PicX;
	int		BaseX;
	int		PicY = 8;
	int		ShellStat;

	// store the initial position so that the ammo counts can be drawn correctly
	PicX = vid.width2D - 50;

	// regular weapons
	for (i = 0; i < 7; i++)
	{
		if (cl.items & (IT_SHOTGUN << i))
		{
			time = cl.item_gettime[i];
			flashon = (int) ((cl.time - time) * 10);

			if (flashon >= 10)
			{
				if (cl.stats[STAT_ACTIVEWEAPON] == (IT_SHOTGUN << i))
					flashon = 1;
				else flashon = 0;
			}
			else flashon = (flashon % 5) + 2;

			// right-align the pictures
			Draw_Pic (PicX - sb_weapons[0][i]->width, PicY, sb_weapons[flashon][i]);
		}

		// skip to the next row, even if it isn't drawn
		PicY += 17;
	}

	// hipnotic stupid fucking weapons
	if (hipnotic)
	{
		int grenadeflashing = 0;

		for (i = 0; i < 4; i++)
		{
			if (cl.items & (1 << hipweapons[i]))
			{
				time = cl.item_gettime[hipweapons[i]];
				flashon = (int) ((cl.time - time) * 10);

				if (flashon >= 10)
				{
					if (cl.stats[STAT_ACTIVEWEAPON] == (1 << hipweapons[i]))
						flashon = 1;
					else flashon = 0;
				}
				else flashon = (flashon % 5) + 2;

				// check grenade launcher
				if (i == 2)
				{
					if (cl.items & HIT_PROXIMITY_GUN)
					{
						if (flashon)
						{
							grenadeflashing = 1;
							Draw_Pic (PicX - hsb_weapons[0][2]->width, 76, hsb_weapons[flashon][2]);
						}
					}
				}
				else if (i == 3)
				{
					if (cl.items & (IT_SHOTGUN << 4))
					{
						if (flashon && !grenadeflashing)
							Draw_Pic (PicX - hsb_weapons[0][3]->width, 76, hsb_weapons[flashon][3]);
						else if (!grenadeflashing)
							Draw_Pic (PicX - hsb_weapons[0][3]->width, 76, hsb_weapons[0][3]);
					}
					else Draw_Pic (PicX - hsb_weapons[0][4]->width, 76, hsb_weapons[flashon][4]);
				}
				else
				{
					if (i == 1 && flashon)
						Draw_String (vid.width2D / 2 - 4 * strlen ("Stupid Fucking Hammer"), vid.height2D - 56, "Stupid Fucking Hammer");

					// right-align the pictures
					Draw_Pic (PicX - sb_weapons[0][i]->width, PicY, hsb_weapons[flashon][i]);
				}
			}

			// skip to next row
			PicY += 17;
		}
	}

    // rogue powered up weapons.
	if (rogue)
		if (cl.stats[STAT_ACTIVEWEAPON] >= RIT_LAVA_NAILGUN)
			for (i = 0; i < 5; i++)
				if (cl.stats[STAT_ACTIVEWEAPON] == (RIT_LAVA_NAILGUN << i))
					Draw_Pic (PicX - rsb_weapons[i]->width, 42 + i * 17, rsb_weapons[i]);

	// ammo counts
	for (i = 0; i < (hipnotic ? 9 : 7); i++)
	{
		// integer division round-off is our friend
		ShellStat = i / 2;

		// cap
		if (ShellStat > 3) ShellStat = 3;

		// no ammo
		if (!cl.stats[STAT_SHELLS + ShellStat]) continue;

		// make the number
		sprintf (num, "%3i", cl.stats[STAT_SHELLS + ShellStat]);

		// convert the colour
		if (num[0] != ' ') num[0] = 18 + num[0] - '0';
		if (num[1] != ' ') num[1] = 18 + num[1] - '0';
		if (num[2] != ' ') num[2] = 18 + num[2] - '0';

		// draw it - center point of the picture
		Draw_String (vid.width2D - 42, 12 + (17 * i), num);
	}
}

//=============================================================================

/*
===============
HUD_DrawHealth
===============
*/
void HUD_DrawHealth (void)
{
	Sbar_DrawNum (50, SbarOffset, cl.stats[STAT_HEALTH], 3, cl.stats[STAT_HEALTH] <= 25);
}


void HUD_DrawArmor (void)
{
	qboolean armoricon = true;
	int ArmX = 50;
	int ArmY = vid.height2D - 72;

	// here we only determine if an icon is needed, and base whether or not we draw the number on that.
	// the icon itself has been replaced.
	if (cl.items & IT_INVULNERABILITY)
		Sbar_DrawNum (ArmX, ArmY, 666, 3, 1);
	else
	{
		if (rogue)
		{
			if (cl.items & RIT_ARMOR3)
				;
			else if (cl.items & RIT_ARMOR2)
				;
			else if (cl.items & RIT_ARMOR1)
				;
			else armoricon = false;
		}
		else
		{
			if (cl.items & IT_ARMOR3)
				;
			else if (cl.items & IT_ARMOR2)
				;
			else if (cl.items & IT_ARMOR1)
				;
			else armoricon = false;
		}

		// only draw the stat number if there's an icon
		if (armoricon) Sbar_DrawNum (ArmX, ArmY, cl.stats[STAT_ARMOR], 3, cl.stats[STAT_ARMOR] <= 25);
	}
}


float XHairS[8] = {0, 0.25, 0.5, 0.75, 0, 0.25, 0.5, 0.75};
float XHairT[8] = {0, 0, 0, 0, 0.5, 0.5, 0.5, 0.5};
float XHairSAdd = 0.25;
float XHairTAdd = 0.5;

// standard 8-bit binary table in reverse with 0/0/0 reversed
float XHairR[] = {1, 0, 1, 0, 1, 0, 1};
float XHairG[] = {1, 1, 0, 0, 1, 1, 0};
float XHairB[] = {1, 1, 1, 1, 0, 0, 0};

void HUD_DrawAmmo (void)
{
	qboolean AmmoData = true;
	int activecrosshair = crosshair.value;
	int activecolour = crosshaircolor.value;
	float xhvlr[2];
	float xhvtb[2];
	int AmmoBoxX = vid.width2D - 40;

	// similar to armour, here we only use the icon to decide if we draw a counter and/or crosshair
	if (rogue)
	{
		if (cl.items & RIT_SHELLS)
			;
		else if (cl.items & RIT_NAILS)
			;
		else if (cl.items & RIT_ROCKETS)
			;
		else if (cl.items & RIT_CELLS)
			;
		else if (cl.items & RIT_LAVA_NAILS)
			;
		else if (cl.items & RIT_PLASMA_AMMO)
			;
		else if (cl.items & RIT_MULTI_ROCKETS)
			;
		else AmmoData = false;
	}
	else
	{
		if (cl.items & IT_SHELLS)
			;
		else if (cl.items & IT_NAILS)
			;
		else if (cl.items & IT_ROCKETS)
			;
		else if (cl.items & IT_CELLS)
			;
		else AmmoData = false;
	}

	// don;t draw a counter or crosshair if there is no icon
	if (!AmmoData) return;

	// ammo counter
	Sbar_DrawNum (AmmoBoxX - 80, SbarOffset, cl.stats[STAT_AMMO], 3, cl.stats[STAT_AMMO] <= 10);

	// check for the stupid fucking hammer
	if (hipnotic && cl.stats[STAT_ACTIVEWEAPON] == 128)
	{
		// stupid fucking hammer
		return;
	}

	// draw the crosshair
	// reduce to a valid scale
	while (activecrosshair > 8) activecrosshair -= 8;

	// no crosshair
	if (activecrosshair < 1) return;

	// make it valid for array indexing
	activecrosshair -= 1;

	// scale colour to valid also
	while (activecolour > 6) activecolour -= 6;
	while (activecolour < 0) activecolour += 6;

	// set up verts
	xhvlr[0] = vid.width2D / 2 - 12 + cl_crossx.value;
	xhvlr[1] = xhvlr[0] + 24;
	xhvtb[0] = vid.height2D / 2 - 12 + cl_crossy.value;
	xhvtb[1] = xhvtb[0] + 24;

	// switch blend func for alphamask texes
	glBlendFunc (GL_SRC_ALPHA, GL_ONE);
	glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	// set colour
	glColor3f (XHairR[activecolour], XHairG[activecolour], XHairB[activecolour]);

	// bind crosshair font texture
	glBindTexture (GL_TEXTURE_2D, r_crosshairtexture.texnum);

	glBegin (GL_QUADS);

	glTexCoord2f (XHairS[activecrosshair], XHairT[activecrosshair]);
	glVertex2f (xhvlr[0], xhvtb[0]);

	glTexCoord2f (XHairS[activecrosshair], XHairT[activecrosshair] + XHairTAdd);
	glVertex2f (xhvlr[0], xhvtb[1]);

	glTexCoord2f (XHairS[activecrosshair] + XHairSAdd, XHairT[activecrosshair] + XHairTAdd);
	glVertex2f (xhvlr[1], xhvtb[1]);

	glTexCoord2f (XHairS[activecrosshair] + XHairSAdd, XHairT[activecrosshair]);
	glVertex2f (xhvlr[1], xhvtb[0]);

	glEnd ();

	// restore state
	glColor3f (1, 1, 1);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
}


int Hud_DrawIndividualBit (int x, int y, qpic_t *pic)
{
	// decrement before drawing so that both x and y pos are correct for the current pic
	x -= (4 + pic->width);
	y -= (pic->height >> 1);

	// now draw
	Draw_Pic (x, y, pic);

	// return the new x pos for use by the next pic
	return x;
}

void HUD_DrawBits (void)
{
	/*
	int i;
	int BitPosX = vid.width2D - 8;
	int BitPosY = vid.height2D - 48;

	// sigilsses
	for (i = 0; i < 4 && !rogue; i++)
		if (cl.items & (1 << (28 + i)))
			BitPosX = Hud_DrawIndividualBit (BitPosX, BitPosY, sb_sigil[i]);

	// regular items
	for (i = 0; i < 6; i++)
		if (cl.items & (1 << (17 + i)))
			BitPosX = Hud_DrawIndividualBit (BitPosX, BitPosY, sb_items[i]);

	// hipnotic items
	for (i = 0; i < 2 && hipnotic; i++)
		if (cl.items & (1 << (24 + i)))
			BitPosX = Hud_DrawIndividualBit (BitPosX, BitPosY, hsb_items[i]);

	// new rogue items
	for (i = 0; i < 2 && rogue; i++)
		if (cl.items & (1 << (29 + i)))
			BitPosX = Hud_DrawIndividualBit (BitPosX, BitPosY, rsb_items[i]);
	*/
}


/*
================
R_DrawHUDModel

Draws a 3D model to the HUD
================
*/
void R_DrawHUDModel (int DisplayListNum, entity_t *e, float scale)
{
    glPushMatrix ();

	// apply modelview transformations
	glTranslatef (e->origin[0], -e->origin[1], e->origin[2]);
	glRotatef (e->angles[1],  0, 0, 1);
	glRotatef (-e->angles[0],  0, 1, 0);
	glRotatef (e->angles[2],  1, 0, 0);

	glScalef (scale, scale, scale);

	// call the display list
	glCallList (DisplayListNum);

	glPopMatrix ();
}


void HUD_Draw3DArmour (void)
{
	int skinnum = 0;
	int Armour;

	entity_t e;

	// HUD replacement for Sbar_DrawArmor
	if (cl.items & IT_INVULNERABILITY && !cl.stats[STAT_ARMOR])
	{
		// HACK - the invulnerability is normally drawn as a powerup widget, but if we 
		// don't have any armour equipped, we'll have an armour amount but no widget for 
		// it, so let's use the invulnerability widget to keep things looking nice.
		if (!HUD_Invul) return;

		Armour = HUD_Invul;
		skinnum = 0;
	}
	else if (!cl.stats[STAT_ARMOR])
	{
		// no armour equipped
		return;
	}
	else
	{
		if (rogue)
		{
			// rogue has different item defs
			if (cl.items & RIT_ARMOR3)
				skinnum = 2;
			else if (cl.items & RIT_ARMOR2)
				skinnum = 1;
			else if (cl.items & RIT_ARMOR1)
				skinnum = 0;
		}
		else
		{
			if (cl.items & IT_ARMOR3)
				skinnum = 2;
			else if (cl.items & IT_ARMOR2)
				skinnum = 1;
			else if (cl.items & IT_ARMOR1)
				skinnum = 0;
		}

		if (!HUD_Arm[skinnum]) return;

		Armour = HUD_Arm[skinnum];
	}

	// set up a temp dummy entity to hold the model
	memset (&e, 0, sizeof (e));

	e.origin[1] = 28;
	e.origin[2] = 60;

	e.angles[1] = bobjrotate;

	// and draw it
	R_DrawHUDModel (Armour, &e, 1.0f);
}


extern int worldtype;

int HUD_DrawIndividual3DBit (int BitNum, int BitPos)
{
	entity_t e;

	// switch the keys
	if (BitNum >= 4)
	{
		switch (worldtype)
		{
		case 1:
			// metal
			BitNum += 2;
			break;
		case 2:
			// base
			break;
		case 0:
		default:
			// medieval
			BitNum += 4;
			break;
		}
	}

	if (!HUD_BitModels[BitNum]) return BitPos;

	// set up a temp dummy entity to hold the model
	memset (&e, 0, sizeof (e));

	e.origin[1] = BitPos;
	e.origin[2] = 54;

	// give each bit a different rotation
	e.angles[1] = bobjrotate + (15 * (BitNum + 1));

	R_DrawHUDModel (HUD_BitModels[BitNum], &e, 0.75f);

	return BitPos - 20;
}


void HUD_Draw3DBits (void)
{
	int BitPos = vid.width2D - 30;

	// keys
	if (cl.items & IT_KEY1) BitPos = HUD_DrawIndividual3DBit (4, BitPos);
	if (cl.items & IT_KEY2) BitPos = HUD_DrawIndividual3DBit (5, BitPos);

	// sigilses
	if (!rogue)
	{
		if (cl.items & IT_SIGIL1) BitPos = HUD_DrawIndividual3DBit (0, BitPos);
		if (cl.items & IT_SIGIL2) BitPos = HUD_DrawIndividual3DBit (1, BitPos);
		if (cl.items & IT_SIGIL3) BitPos = HUD_DrawIndividual3DBit (2, BitPos);
		if (cl.items & IT_SIGIL4) BitPos = HUD_DrawIndividual3DBit (3, BitPos);
	}
}

/*
====================
SbarDraw3DModels

MH - draws 3D models for a lot of the little status bar widgets

All 3D widgets aside from health and ammo are drawn here.  Health and ammo require a viewport
with a perspective projection so as to get them looking right, so they need to be done
before any of the other 2D stuff is drawn (otherwise we have an utter mess).
====================
*/
void SbarDraw3DModels (void)
{
	int i;

	bobjrotate = anglemod (100 * realtime);

	// initialize our matrices
	// these are pushed/popped so as not to interfere with the current matrices, whatever they may be
	glMatrixMode (GL_PROJECTION);
	glPushMatrix ();
	glLoadIdentity ();

	// flip the orthographic projection along both axes for the 3D stuff
	glOrtho  (0, vid.width2D, 0, vid.height2D, -400, 400);

	glMatrixMode (GL_MODELVIEW);
	glPushMatrix ();
	glLoadIdentity ();

	// put Z going up and X going deep to match Quakes main render
	// Y remains at right to left, Z becomes top to bottom and X becomes near to far
	glRotatef (-90, 1, 0, 0);
	glRotatef (90, 0, 0, 1);

	// hack the depth range to prevent sbar models being clipped by world geometry
	glEnable (GL_DEPTH_TEST);
	glDepthRange (0, 0.1);

	// draw the models
	HUD_Draw3DArmour ();
	//HUD_Draw3DWeapons ();
	//HUD_Draw3DPowerups ();
	HUD_Draw3DBits ();

	glMatrixMode (GL_PROJECTION);
	glPopMatrix ();

	glMatrixMode (GL_MODELVIEW);
	glPopMatrix ();

	// restore the previous depth range
	glDepthRange (0.3, 1);
	glDisable (GL_DEPTH_TEST);
}


void HUD_DrawFaceBSP (int x, int y, int size)
{
	entity_t e;

	if (!HUD_Face) return;

	glViewport (x, y, size, size);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	// use constant fov to keep perspective consistent across different aspect ratios.  this viewport 
	// has an aspect ratio of 1 so use fov 90 (same as quake in a window with aspect ratio of 1)
	gluPerspective (90, 1, 4, 150);

	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();

	glEnable (GL_DEPTH_TEST);
	glDepthRange (0, 0.1);

	// put Z going up and X going deep to match Quakes main render
	// Y remains at right to left, Z becomes top to bottom and X becomes near to far
	glRotatef (-90, 1, 0, 0);
	glRotatef (90, 0, 0, 1);

	// set up a temp dummy entity to hold the model
	memset (&e, 0, sizeof (e));

	// set origin and angles for any transforms we might want to do
	e.origin[0] = 35;		// always positive, higher numbers are deeper
	e.origin[1] = 0;		// positive = to the left, negative = to the right
	e.origin[2] = -18;		// negative = further down, positive = higher up

	e.angles[1] = anglemod (25 * realtime); //bobjrotate;

	glPushMatrix ();
	glRotatef (-17.5, 0, 1, 0);

	// and draw it
	R_DrawHUDModel (HUD_Face, &e, 1.0f);

	glPopMatrix ();

	// restore the previous depth range
	glDepthRange (0.3, 1);
	glDisable (GL_DEPTH_TEST);
}


void HUD_DrawAmmoBSP (int x, int y, int size)
{
	entity_t e;
	int AmmoNum;

	// HUD Replacement for Sbar_DrawAmmo
	if (rogue)
	{
		if (cl.items & RIT_SHELLS)
			AmmoNum = 0;
		else if (cl.items & RIT_NAILS)
			AmmoNum = 1;
		else if (cl.items & RIT_ROCKETS)
			AmmoNum = 2;
		else if (cl.items & RIT_CELLS)
			AmmoNum = 3;
		else if (cl.items & RIT_LAVA_NAILS)
			AmmoNum = 4;
		else if (cl.items & RIT_PLASMA_AMMO)
			AmmoNum = 5;
		else if (cl.items & RIT_MULTI_ROCKETS)
			AmmoNum = 6;
		else return;
	}
	else
	{
		if (cl.items & IT_SHELLS)
			AmmoNum = 0;
		else if (cl.items & IT_NAILS)
			AmmoNum = 1;
		else if (cl.items & IT_ROCKETS)
			AmmoNum = 2;
		else if (cl.items & IT_CELLS)
			AmmoNum = 3;
		else return;
	}

	if (!HUD_Ammo[AmmoNum]) return;

	glViewport (x, y, size, size);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	// use constant fov to keep perspective consistent across different aspect ratios.  this viewport 
	// has an aspect ratio of 1 so use fov 90 (same as quake in a window with aspect ratio of 1)
	gluPerspective (90, 1, 4, 150);

	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();

	glEnable (GL_DEPTH_TEST);
	glDepthRange (0, 0.1);

	// put Z going up and X going deep to match Quakes main render
	// Y remains at right to left, Z becomes top to bottom and X becomes near to far
	glRotatef (-90, 1, 0, 0);
	glRotatef (90, 0, 0, 1);

	// set up a temp dummy entity to hold the model
	memset (&e, 0, sizeof (e));

	// set origin and angles for any transforms we might want to do
	e.origin[0] = 35;		// always positive, higher numbers are deeper
	e.origin[1] = 0;		// positive = to the left, negative = to the right
	e.origin[2] = -18;		// negative = further down, positive = higher up

	e.angles[1] = anglemod (25 * realtime); //bobjrotate;

	glPushMatrix ();
	glRotatef (-17.5, 0, 1, 0);

	// and draw it
	R_DrawHUDModel (HUD_Ammo[AmmoNum], &e, 1.0f);

	glPopMatrix ();

	// restore the previous depth range
	glDepthRange (0.3, 1);
	glDisable (GL_DEPTH_TEST);
}


#define HUD_VP_WIDTH 54
void GL_Set2D (void);

void SbarDraw3DBrushes (void)
{
	int HUDVPSize = (HUD_VP_WIDTH * glwidth) / vid.width2D;

	// these are drawn in 2 seperate viewports so they must be done seperately
	// take your pick for the order...
	HUD_DrawFaceBSP (0, 0, HUDVPSize);
	HUD_DrawAmmoBSP (glwidth - HUDVPSize, 0, HUDVPSize);

	// restore 2D view
	GL_Set2D ();
}



/*
===============
Sbar_Draw
===============
*/
void Sbar_Draw (void)
{
	// only used for cases where mods use this as a hack to indicate cutscenes.
	if (scr_viewsize.value == 120) return;

	// no sbar in intermission
	if (cl.intermission) return;

	// console is full screen
	if (scr_con_current == vid.height2D) return;

	// not connected
	if (cls.state != ca_connected) return;

	SbarOffset = vid.height2D - 32;

	if (cl.stats[STAT_HEALTH] <= 0)
	{
		Sbar_DrawScoreboard ();
		return;
	}

	// scoreboard
	if (sb_showscores) Sbar_DrawScoreboard ();

	// 3d models
	SbarDraw3DBrushes ();
	SbarDraw3DModels ();

	// armor
	HUD_DrawArmor ();

	// face
	HUD_DrawHealth ();

	// ammo icon
	HUD_DrawAmmo ();

	// weaponses
	HUD_DrawWeaponses ();

	// keys and sigils
	HUD_DrawBits ();

	// deathmatch overlay
	if (cl.gametype == GAME_DEATHMATCH && !sb_showscores)
		Sbar_MiniDeathmatchOverlay ();
}

//=============================================================================

/*
==================
Sbar_IntermissionNumber

==================
*/
void Sbar_IntermissionNumber (int x, int y, int num, int digits, int color)
{
	char			str[12];
	char			*ptr;
	int				l, frame;

	l = Sbar_itoa (num, str);
	ptr = str;
	if (l > digits)
		ptr += (l-digits);
	if (l < digits)
		x += (digits-l)*24;

	while (*ptr)
	{
		if (*ptr == '-')
			frame = STAT_MINUS;
		else
			frame = *ptr -'0';

		Draw_TransPic (x,y,sb_nums[color][frame]);
		x += 24;
		ptr++;
	}
}

/*
==================
Sbar_DeathmatchOverlay

==================
*/
void Sbar_DeathmatchOverlay (void)
{
	qpic_t			*pic;
	int				i, k, l;
	int				top, bottom;
	int				x, y, f;
	char			num[12];
	scoreboard_t	*s;

	pic = Draw_CachePic ("gfx/ranking.lmp");
	M_DrawPic ((320-pic->width)/2, 8, pic);

// scores
	Sbar_SortFrags ();

// draw the text
	l = scoreboardlines;

	x = 80 + ((vid.width2D - 320)>>1);
	y = 40;
	for (i=l - 1 ; i >= 0 ; i--)
	{
		k = fragsort[i];
		s = &cl.scores[k];
		if (!s->name[0])
			continue;

	// draw background
		top = s->colors & 0xf0;
		bottom = (s->colors & 15)<<4;
		top = Sbar_ColorForMap (top);
		bottom = Sbar_ColorForMap (bottom);

		Draw_Fill ( x, y, 40, 4, top);
		Draw_Fill ( x, y+4, 40, 4, bottom);

	// draw number
		f = s->frags;
		sprintf (num, "%3i",f);

		Draw_Character ( x+8 , y, num[0]);
		Draw_Character ( x+16 , y, num[1]);
		Draw_Character ( x+24 , y, num[2]);

		if (k == cl.viewentity - 1)
			Draw_Character ( x - 8, y, 12);

#if 0
{
	int				total;
	int				n, minutes, tens, units;

	// draw time
		total = cl.completed_time - s->entertime;
		minutes = (int)total/60;
		n = total - minutes*60;
		tens = n/10;
		units = n%10;

		sprintf (num, "%3i:%i%i", minutes, tens, units);

		Draw_String ( x+48 , y, num);
}
#endif

	// draw name
		Draw_String (x+64, y, s->name);

		y += 10;
	}
}

/*
==================
Sbar_DeathmatchOverlay

==================
*/
void Sbar_MiniDeathmatchOverlay (void)
{
	int				i, k, l;
	int				top, bottom;
	int				x, y, f;
	char			num[12];
	scoreboard_t	*s;
	int				numlines;
	int				sb_lines = 24;

	// scores
	Sbar_SortFrags ();

	// draw the text
	y = vid.height2D - 96;
	x = 8;

	for (i = 0; i < scoreboardlines && y > 12; i++)
	{
		// get current entry
		k = fragsort[i];
		s = &cl.scores[k];

		// no name!
		if (!s->name[0]) continue;

		// draw background
		top = s->colors & 0xf0;
		bottom = (s->colors & 15) << 4;
		top = Sbar_ColorForMap (top);
		bottom = Sbar_ColorForMap (bottom);

		Draw_Fill (x, y + 1, 40, 3, top);
		Draw_Fill (x, y + 4, 40, 4, bottom);

		// draw number
		f = s->frags;
		sprintf (num, "%3i", f);

		Draw_Character (x + 8, y, num[0]);
		Draw_Character (x + 16, y, num[1]);
		Draw_Character (x + 24, y, num[2]);

		// mark us
		if (k == cl.viewentity - 1)
		{
			Draw_Character (x, y, 16);
			Draw_Character (x + 32, y, 17);
		}

		// draw name
		Draw_String (x + 48, y, s->name);

		y -= 12;
	}
}

/*
==================
Sbar_IntermissionOverlay

==================
*/
void Sbar_IntermissionOverlay (void)
{
	qpic_t	*pic;
	int		dig;
	int		num;

	if (cl.gametype == GAME_DEATHMATCH)
	{
		Sbar_DeathmatchOverlay ();
		return;
	}

	pic = Draw_CachePic ("gfx/complete.lmp");
	Draw_Pic (64, 24, pic);

	pic = Draw_CachePic ("gfx/inter.lmp");
	Draw_TransPic (0, 56, pic);

// time
	dig = cl.completed_time/60;
	Sbar_IntermissionNumber (160, 64, dig, 3, 0);
	num = cl.completed_time - dig*60;
	Draw_TransPic (234,64,sb_colon);
	Draw_TransPic (246,64,sb_nums[0][num/10]);
	Draw_TransPic (266,64,sb_nums[0][num%10]);

	Sbar_IntermissionNumber (160, 104, cl.stats[STAT_SECRETS], 3, 0);
	Draw_TransPic (232,104,sb_slash);
	Sbar_IntermissionNumber (240, 104, cl.stats[STAT_TOTALSECRETS], 3, 0);

	Sbar_IntermissionNumber (160, 144, cl.stats[STAT_MONSTERS], 3, 0);
	Draw_TransPic (232,144,sb_slash);
	Sbar_IntermissionNumber (240, 144, cl.stats[STAT_TOTALMONSTERS], 3, 0);

}


/*
==================
Sbar_FinaleOverlay

==================
*/
void Sbar_FinaleOverlay (void)
{
	qpic_t	*pic;

	pic = Draw_CachePic ("gfx/finale.lmp");
	Draw_TransPic ( (vid.width2D-pic->width)/2, 16, pic);
}
